/***************************************************************
 * Name:      wximagepanel.cpp
 * Purpose:   ...
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2011-01-26
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
#include <wx/config.h>
#include <wx/hashmap.h>
#include <wx/statbmp.h>

// Private headers
#include "wxextthread.h"
#include "wximagepanel.h"


//////////////////////////////////////////////////////////
//  class wxImagePanelThread
//
/// \brief ...
///
/// ...
class wxImagePanelThread : public wxExtThread
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poEvtHandler             a pointer to the event handler that owns the thread.
    /// \param nEventInt                ...
    wxImagePanelThread( wxEvtHandler *poEvtHandler, int nEventInt )
      : wxExtThread( poEvtHandler, nEventInt )
    {
    }

    /// \brief Thread execution starts here.
    virtual ExitCode Entry()
    {
        // ...
        ( (wxImagePanel *)m_poEvtHandler )->OnThreadEntry( m_nEventInt );

        // Return with exit code 0 (success).
        return 0;
    }
};


//////////////////////////////////////////////////////////
// Class wxImagePanel
//
// Event table
//
BEGIN_EVENT_TABLE( wxImagePanel, wxScrolledWindow )
    EVT_MENU( wxExtThread::ID_EXIT, wxImagePanel::OnThreadExit )
END_EVENT_TABLE()


// Standard constructor
//
wxImagePanel::wxImagePanel( wxWindow *poParent, wxWindowID nId,
                            const wxPoint &oPos, const wxSize &oSize,
                            long nStyle, const wxString& strName )
: wxScrolledWindow( poParent, nId, oPos, oSize, nStyle, strName )
{
    // Empty the panel.
    Reset();

    // Reset window settings.
    wxConfigBase *poConfig = wxConfigBase::Get();
    poConfig->SetRecordDefaults();
    poConfig->SetPath( wxT( "/" )  );
    poConfig->SetPath( GetName() );

    // Reset bitmap and spacer size.
    m_nPanelPagesBitmapSize= poConfig->Read( wxT( "PanelPagesBitmapSize" ), 96 );
    m_nPanelPagesBitmapSpace= poConfig->Read( wxT( "PanelPagesBitmapSpace" ), 16 );
}

// Virtual destructor
//
wxImagePanel::~wxImagePanel()
{
    // Save configuration data.
    wxConfigBase *poConfig= wxConfigBase::Get();

    poConfig->SetPath( wxT( "/" ) );
    poConfig->SetPath( GetName() );

    // Save bitmap and spacer size.
    poConfig->Write( wxT( "PanelPagesBitmapSize" ), (long)m_nPanelPagesBitmapSize );
    poConfig->Write( wxT( "PanelPagesBitmapSpace" ), (long)m_nPanelPagesBitmapSpace );
}

// Add images to the panel.
//
void wxImagePanel::AddImage( wxImage oImage )
{
    m_oImageHash[m_nPanelPagesCount]= oImage;
    wxImagePanelThread * poThread=new wxImagePanelThread( this, m_nPanelPagesCount );

    poThread->Create();
    poThread->Run();
    m_nPanelPagesCount++;
}

// ...
//
void wxImagePanel::OnThreadEntry( int nId )
{
    wxImage oImage= m_oImageHash[nId];

    double xfactor= (double)m_nPanelPagesBitmapSize
                    / (double)( ( oImage.GetWidth() > oImage.GetHeight() ) ? oImage.GetWidth() : oImage.GetHeight() );

    if( xfactor < 1.0 )
    {
        oImage.Rescale( (int)( (double)oImage.GetWidth()  * xfactor ),
                        (int)( (double)oImage.GetHeight() * xfactor ),
                        wxIMAGE_QUALITY_HIGH );
        m_oImageHash[nId]= oImage;
    }
}


// Handle ...
//
void wxImagePanel::OnThreadExit( wxCommandEvent& oEvent )
{
    int nId= oEvent.GetInt();

    wxImage oImage= m_oImageHash[nId];

    m_oImageHash.erase( nId );

    wxPoint oStartingPoint( m_nPanelPagesBitmapSpace,
                            m_nPanelPagesBitmapSpace + ( m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize ) * nId );

    SetScrollbars( 1, 1,
                   m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize + m_nPanelPagesBitmapSpace,
                   m_nPanelPagesBitmapSpace + ( m_nPanelPagesBitmapSpace + m_nPanelPagesBitmapSize ) * m_nPanelPagesCount );

    new wxStaticBitmap( this, wxID_ANY, wxBitmap( oImage ), oStartingPoint );
//    new wxBitmapButton( this, wxID_ANY, wxBitmap( oImage ), oStartingPoint );

//    Scroll( 0, oStartingPoint.y );
}
