/***************************************************************
 * Name:      wximagepanel.h
 * Purpose:   ...
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2011-01-26
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXIMAGEPANEL_H
#define WXIMAGEPANEL_H

//////////////////////////////////////////////////////////
// Hash map wxImageHash (for images)
//
WX_DECLARE_HASH_MAP( int, wxImage, wxIntegerHash, wxIntegerEqual, wxImageHash );


//////////////////////////////////////////////////////////
// Class wxImagePanel
//
/// \brief A panel that can display as list of images
///        (e. g. pages) and scroll.
///
/// ...
class wxImagePanel : public wxScrolledWindow
{
    public:
        /// \brief Standard constructor
        ///
        /// \param poParent     a pointer to the dialog windows's parent window.
        /// \param nId          the window id, default: 1
        /// \param oPos         the window position, default: wxDefaultPosition
        /// \param oSize        the window size, default: wxDefaultSize
        /// \param nStyle       window style (default: wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER)
        /// \param strName      class name (default: wxT( "wxIViewCanvas" ))
        wxImagePanel( wxWindow *poParent,
                       wxWindowID nId= wxID_ANY,
                       const wxPoint &oPos= wxDefaultPosition,
                       const wxSize &oSize= wxDefaultSize,
                       long nStyle= wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER,
                       const wxString& strName= wxT( "wxImagePanel" )
                     );

        /// Virtual destructor
        virtual ~wxImagePanel();

        /// \brief Empty the panel.
        virtual void Reset()
        {
            DestroyChildren();
            SetScrollbars( 1, 1, 1, 1 );
            m_nPanelPagesCount= 0;
        }

        /// \brief Add images to the panel.
        ///
        /// \param oImage       an image to convert to a thumbnail
        ///
        /// Note: I moved all GUI functions from AddImage() to OnThreadExit().
        ///       It should therefore be possible to call AddImage() from another
        ///       (non-GUI-)thread.
        ///
        virtual void AddImage( wxImage oImage );

        /// \brief ...
        ///
        /// \param nId          ...
        virtual void OnThreadEntry( int nId );

    private:
        // Event handlers
        void OnThreadExit( wxCommandEvent& oEvent );  ///< Handle ...

    protected:
         // Member variables
        int m_nPanelPagesCount;             ///< The number of the pages in the pages panel.
        wxImageHash m_oImageHash;           ///< A hash map to get the image corresponding to event id in OnThreadEntry().

        // Options
        int m_nPanelPagesBitmapSize;        ///< ...
        int m_nPanelPagesBitmapSpace;       ///< ...

    private:
        // Event table.
        DECLARE_EVENT_TABLE()
};

#endif // WXISCANMAIN_H

