/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxiviewpaint.h
// Author:      Daniel Nell
// Created:     2005/10/18
// Copyright:   (c) Daniel Nell
// License:     wxWindows license
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXIVIEWPAINT_H
#define __WXIVIEWPAINT_H

/////////////////////////////////////////////////////////////////////////////
//  class wxIViewPaintBase
//
/// \brief Basic printing and print previewing handling.
///
/// This is a helper class for the wxPrintOut implementation
/// done in wxIViewPrintout class.
///
/// Note: This is an abstract class ("pure virtual") and
///       works as an interface description. Its member
///       functions must be implemented in its sub classes.
class wxIViewPaintBase
{
    public:
        /// \brief Calculates the page extents in mm while
        ///        printing or print previewing
        ///
        /// \param pnWidth      a pointer to an integer for
        ///                     returning the page width in mm
        /// \param pnHeight     a pointer to an integer for
        ///                     returning the page height in mm
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight ) = 0;

        /// Get the highest page number of the document to print.
        virtual int  GetMaxPage() = 0;

        /// \brief Switch to the given page number to print.
        ///
        /// \param nPage        the page number
        virtual bool GotoPage( int nPage ) = 0;

        /// \brief Show (print)...
        ///
        /// \param oDc          the device context to print on.
        virtual bool Paint( wxDC& oDc ) = 0;

    protected:
        /// Virtual destructor.
        virtual ~wxIViewPaintBase(){}
};

#endif // __WXIVIEWPAINT_H
